@@ -2,7 +2,7 @@
 #
 # The MIT License (MIT)
 #
-# Copyright (c) Tavendo GmbH
+# Copyright (c) Crossbar.io Technologies GmbH
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -97,7 +97,7 @@ class TestUris(unittest.TestCase):
                   u"hello\xE2\x82\xACworld..foo",
                   u"hello\xF0\xA4\xAD\xA2world..foo",
                   ]:
-            self.assertEqual(u, message.check_or_raise_uri(u, allowEmptyComponents=True))
+            self.assertEqual(u, message.check_or_raise_uri(u, allow_empty_components=True))
 
     def test_invalid_uris_loose_empty(self):
         for u in [0,
@@ -113,7 +113,7 @@ class TestUris(unittest.TestCase):
                   u"com.myapp.product#",
                   u"com.#.product",
                   ]:
-            self.assertRaises(ProtocolError, message.check_or_raise_uri, u, allowEmptyComponents=True)
+            self.assertRaises(ProtocolError, message.check_or_raise_uri, u, allow_empty_components=True)
 
     def test_valid_uris_strict_nonempty(self):
         for u in [u"com.myapp.topic1",
@@ -156,7 +156,7 @@ class TestUris(unittest.TestCase):
                   u".",
                   u"",
                   ]:
-            self.assertEqual(u, message.check_or_raise_uri(u, strict=True, allowEmptyComponents=True))
+            self.assertEqual(u, message.check_or_raise_uri(u, strict=True, allow_empty_components=True))
 
     def test_invalid_uris_strict_empty(self):
         for u in [0,
@@ -178,7 +178,7 @@ class TestUris(unittest.TestCase):
                   u"hello\xE2\x82\xACworld..foo",
                   u"hello\xF0\xA4\xAD\xA2world..foo",
                   ]:
-            self.assertRaises(ProtocolError, message.check_or_raise_uri, u, strict=True, allowEmptyComponents=True)
+            self.assertRaises(ProtocolError, message.check_or_raise_uri, u, strict=True, allow_empty_components=True)
 
 
 class TestErrorMessage(unittest.TestCase):
@@ -262,6 +262,38 @@ class TestSubscribeMessage(unittest.TestCase):
         self.assertEqual(msg.match, message.Subscribe.MATCH_PREFIX)
         self.assertEqual(msg.marshal(), wmsg)
 
+    def test_get_retained_default_false(self):
+        wmsg = [message.Subscribe.MESSAGE_TYPE, 123456, {u'match': u'prefix'}, u'com.myapp.topic1']
+        msg = message.Subscribe.parse(wmsg)
+        self.assertIsInstance(msg, message.Subscribe)
+        self.assertEqual(msg.request, 123456)
+        self.assertEqual(msg.topic, u'com.myapp.topic1')
+        self.assertEqual(msg.get_retained, None)
+        self.assertNotEqual(msg.get_retained, True)
+        self.assertEqual(msg.match, message.Subscribe.MATCH_PREFIX)
+        self.assertEqual(msg.marshal(), wmsg)
+
+    def test_get_retained_explicit_false(self):
+        wmsg = [message.Subscribe.MESSAGE_TYPE, 123456, {u'match': u'prefix', u'get_retained': False}, u'com.myapp.topic1']
+        msg = message.Subscribe.parse(wmsg)
+        self.assertIsInstance(msg, message.Subscribe)
+        self.assertEqual(msg.request, 123456)
+        self.assertEqual(msg.topic, u'com.myapp.topic1')
+        self.assertEqual(msg.get_retained, False)
+        self.assertNotEqual(msg.get_retained, True)
+        self.assertEqual(msg.match, message.Subscribe.MATCH_PREFIX)
+        self.assertEqual(msg.marshal(), wmsg)
+
+    def test_get_retained_explicit_true(self):
+        wmsg = [message.Subscribe.MESSAGE_TYPE, 123456, {u'match': u'prefix', u'get_retained': True}, u'com.myapp.topic1']
+        msg = message.Subscribe.parse(wmsg)
+        self.assertIsInstance(msg, message.Subscribe)
+        self.assertEqual(msg.request, 123456)
+        self.assertEqual(msg.topic, u'com.myapp.topic1')
+        self.assertEqual(msg.get_retained, True)
+        self.assertEqual(msg.match, message.Subscribe.MATCH_PREFIX)
+        self.assertEqual(msg.marshal(), wmsg)
+
 
 class TestSubscribedMessage(unittest.TestCase):
 
@@ -371,12 +403,12 @@ class TestPublishMessage(unittest.TestCase):
         self.assertEqual(msg[4], [1, 2, 3])
         self.assertEqual(msg[5], {u'foo': 23, u'bar': u'hello'})
 
-        e = message.Publish(123456, u'com.myapp.topic1', exclude_me=False, exclude=[300], eligible=[100, 200, 300], disclose_me=True)
+        e = message.Publish(123456, u'com.myapp.topic1', exclude_me=False, exclude=[300], eligible=[100, 200, 300])
         msg = e.marshal()
         self.assertEqual(len(msg), 4)
         self.assertEqual(msg[0], message.Publish.MESSAGE_TYPE)
         self.assertEqual(msg[1], 123456)
-        self.assertEqual(msg[2], {u'exclude_me': False, u'disclose_me': True, u'exclude': [300], u'eligible': [100, 200, 300]})
+        self.assertEqual(msg[2], {u'exclude_me': False, u'exclude': [300], u'eligible': [100, 200, 300]})
         self.assertEqual(msg[3], u'com.myapp.topic1')
 
     def test_parse_and_marshal(self):
@@ -390,7 +422,6 @@ class TestPublishMessage(unittest.TestCase):
         self.assertEqual(msg.exclude_me, None)
         self.assertEqual(msg.exclude, None)
         self.assertEqual(msg.eligible, None)
-        self.assertEqual(msg.disclose_me, None)
         self.assertEqual(msg.marshal(), wmsg)
 
         wmsg = [message.Publish.MESSAGE_TYPE, 123456, {}, u'com.myapp.topic1', [1, 2, 3], {u'foo': 23, u'bar': u'hello'}]
@@ -403,10 +434,9 @@ class TestPublishMessage(unittest.TestCase):
         self.assertEqual(msg.exclude_me, None)
         self.assertEqual(msg.exclude, None)
         self.assertEqual(msg.eligible, None)
-        self.assertEqual(msg.disclose_me, None)
         self.assertEqual(msg.marshal(), wmsg)
 
-        wmsg = [message.Publish.MESSAGE_TYPE, 123456, {u'exclude_me': False, u'disclose_me': True, u'exclude': [300], u'eligible': [100, 200, 300]}, u'com.myapp.topic1']
+        wmsg = [message.Publish.MESSAGE_TYPE, 123456, {u'exclude_me': False, u'exclude': [300], u'eligible': [100, 200, 300]}, u'com.myapp.topic1']
         msg = message.Publish.parse(wmsg)
         self.assertIsInstance(msg, message.Publish)
         self.assertEqual(msg.request, 123456)
@@ -416,7 +446,39 @@ class TestPublishMessage(unittest.TestCase):
         self.assertEqual(msg.exclude_me, False)
         self.assertEqual(msg.exclude, [300])
         self.assertEqual(msg.eligible, [100, 200, 300])
-        self.assertEqual(msg.disclose_me, True)
+        self.assertEqual(msg.marshal(), wmsg)
+
+    def test_retain_default_false(self):
+        """
+        Retain, when not specified, is False-y by default.
+        """
+        wmsg = [message.Publish.MESSAGE_TYPE, 123456, {u'exclude_me': False, u'exclude': [300], u'eligible': [100, 200, 300]}, u'com.myapp.topic1']
+        msg = message.Publish.parse(wmsg)
+        self.assertIsInstance(msg, message.Publish)
+        self.assertEqual(msg.retain, None)
+        self.assertIsNot(msg.retain, True)
+        self.assertEqual(msg.marshal(), wmsg)
+
+    def test_retain_explicit_false(self):
+        """
+        Retain, when specified as False, shows up in the message.
+        """
+        wmsg = [message.Publish.MESSAGE_TYPE, 123456, {u'exclude_me': False, u'retain': False, u'exclude': [300], u'eligible': [100, 200, 300]}, u'com.myapp.topic1']
+        msg = message.Publish.parse(wmsg)
+        self.assertIsInstance(msg, message.Publish)
+        self.assertEqual(msg.retain, False)
+        self.assertIsNot(msg.retain, True)
+        self.assertEqual(msg.marshal(), wmsg)
+
+    def test_retain_explicit_true(self):
+        """
+        Retain, when specified as True, shows up in the message.
+        """
+        wmsg = [message.Publish.MESSAGE_TYPE, 123456, {u'exclude_me': False, u'retain': True, u'exclude': [300], u'eligible': [100, 200, 300]}, u'com.myapp.topic1']
+        msg = message.Publish.parse(wmsg)
+        self.assertIsInstance(msg, message.Publish)
+        self.assertEqual(msg.retain, True)
+        self.assertIs(msg.retain, True)
         self.assertEqual(msg.marshal(), wmsg)
 
 
@@ -499,6 +561,29 @@ class TestEventMessage(unittest.TestCase):
         self.assertEqual(msg.publisher, 300)
         self.assertEqual(msg.marshal(), wmsg)
 
+    def test_retained_default_false(self):
+        wmsg = [message.Event.MESSAGE_TYPE, 123456, 789123, {}]
+        msg = message.Event.parse(wmsg)
+        self.assertIsInstance(msg, message.Event)
+        self.assertEqual(msg.retained, None)
+        self.assertNotEqual(msg.retained, True)
+        self.assertEqual(msg.marshal(), wmsg)
+
+    def test_retained_explicit_false(self):
+        wmsg = [message.Event.MESSAGE_TYPE, 123456, 789123, {u'retained': False}]
+        msg = message.Event.parse(wmsg)
+        self.assertIsInstance(msg, message.Event)
+        self.assertEqual(msg.retained, False)
+        self.assertNotEqual(msg.retained, True)
+        self.assertEqual(msg.marshal(), wmsg)
+
+    def test_retained_explicit_true(self):
+        wmsg = [message.Event.MESSAGE_TYPE, 123456, 789123, {u'retained': True}]
+        msg = message.Event.parse(wmsg)
+        self.assertIsInstance(msg, message.Event)
+        self.assertEqual(msg.retained, True)
+        self.assertEqual(msg.marshal(), wmsg)
+
 
 class TestRegisterMessage(unittest.TestCase):
 
